#!/usr/bin/perl
#
# install-rl: install script for Reading List system
#
# $Id: install-rl.pl,v 1.5 2003/08/04 14:11:24 jon Exp $
#
#   Copyright (C) 2001-2002 Loughborough University Library
#   Authors: Jon Knight and Gary Brewerton
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

use strict;
use Cwd;
use DBI;
use File::Basename;

# Version
my $Version = "v5.0";

# Defaults for some of our variables
my $thishost = `hostname`;
chomp($thishost);
my $thisdnsdomain = `dnsdomainname` || "lut.ac.uk";
chomp($thisdnsdomain);
my $ReadingListsLibraryDir = "/usr/local/ReadingLists";
my $BackendBinDir = "/usr/local/ReadingLists/bin";
my $ModuleCodeFormat = "YYDDSXXX";
my $OpacURL = "http://opac.$thisdnsdomain/ALEPH";
my $OpacISBNURL = "http://opac.$thisdnsdomain/ALEPH/-/ext-find?base=lbo01&find=WIS=%s";
my $HostURI = "https://$thishost/";
my $Z3950Hostname = "opac.$thisdnsdomain";
my $Z3950Port = 9929;
my $Z3950DBName = "USM01";
my $Z3950Username = "";
my $Z3950Password = "";
my $CSS = "  <link rel='stylesheet' type='text/css' href='/readinglists.css' title='Default'>";
my $CSSPath = "/";
my $CSSDir = "/var/www/html/";
my $UserCGI = "/rl";
my $UserCGIBinDir = "/var/www/cgi-bin/readinglists";
my $AdminCGI = "/rl/admin";
my $AdminCGIBinDir = "/var/www/cgi-bin/readinglistsadmin";
my $LibrarianCGI = "/rl/admin/librarians";
my $LibrarianCGIBinDir = "/var/www/cgi-bin/readinglistslibrarian";
my $DefaultLibraryEmail = "library\@$thisdnsdomain";
my $SystemAdminEmail = "librarysystems\@$thisdnsdomain";
my $DBname = "readinglist";
my $DBuser = "rl";
my $DBpassword = "oghma";

# Over ride defaults with existing package if one exists
my $got_pm = 0;
if ($ARGV[0]) {
    if (-f $ARGV[0]) {
	print "Using settings in $ARGV[0] to override defaults!\n";
	require $ARGV[0];
	$ReadingListsLibraryDir = dirname($ARGV[0]);
        $BackendBinDir = ReadingLists::BackendBinDir();
        $ModuleCodeFormat = ReadingLists::ModuleCodeFormat();
        $OpacURL = ReadingLists::OpacURL();
        $OpacISBNURL = ReadingLists::OpacISBNURL();
        $HostURI = ReadingLists::HostURI();
        $Z3950Hostname = ReadingLists::Z3950Hostname();
        $Z3950Port = ReadingLists::Z3950Port();
        $Z3950DBName = ReadingLists::Z3950DBName();
        $Z3950Username = ReadingLists::Z3950Username();
        $Z3950Password = ReadingLists::Z3950Password();
        $UserCGI = ReadingLists::UserCGI();
        $UserCGIBinDir = ReadingLists::UserCGIBinDir();
        $AdminCGI = ReadingLists::AdminCGI();
        $AdminCGIBinDir = ReadingLists::AdminCGIBinDir();
        $LibrarianCGI = ReadingLists::LibrarianCGI();
        $LibrarianCGIBinDir = ReadingLists::LibrarianCGIBinDir();
        $DefaultLibraryEmail = ReadingLists::DefaultLibraryEmail();
        $SystemAdminEmail = ReadingLists::SystemAdminEmail();
        my $olddsn = ReadingLists::dsn();
        $olddsn =~ /database=(.+);user=(.+);password=(.+);mysql/;
        $DBname = $1;
        $DBuser = $2;
        $DBpassword = $3;
	$got_pm=1;
    } else {
	print "Couldn't read $ARGV[0] to get current settings\n";
	exit;
    }
}
if (!$got_pm && getcwd() ne $ReadingListsLibraryDir && 
    -r "$ReadingListsLibraryDir/ReadingLists.pm") {
    print "\nFound $ReadingListsLibraryDir/ReadingLists.pm at the target " .
	"location.\n";
    if(ask("Would like you like to use this info to override factory " .
	   "defaults?", "y", "q") eq 'y') {
	print "\nUsing this info to override factory defaults!\n";
	require "$ReadingListsLibraryDir/ReadingLists.pm";
        $BackendBinDir = ReadingLists::BackendBinDir();
        $ModuleCodeFormat = ReadingLists::ModuleCodeFormat();
        $OpacURL = ReadingLists::OpacURL();
        $OpacISBNURL = ReadingLists::OpacISBNURL();
        $HostURI = ReadingLists::HostURI();
        $Z3950Hostname = ReadingLists::Z3950Hostname();
        $Z3950Port = ReadingLists::Z3950Port();
        $Z3950DBName = ReadingLists::Z3950DBName();
        $Z3950Username = ReadingLists::Z3950Username();
        $Z3950Password = ReadingLists::Z3950Password();
        $UserCGI = ReadingLists::UserCGI();
        $UserCGIBinDir = ReadingLists::UserCGIBinDir();
        $AdminCGI = ReadingLists::AdminCGI();
        $AdminCGIBinDir = ReadingLists::AdminCGIBinDir();
        $LibrarianCGI = ReadingLists::LibrarianCGI();
        $LibrarianCGIBinDir = ReadingLists::LibrarianCGIBinDir();
        $DefaultLibraryEmail = ReadingLists::DefaultLibraryEmail();
        $SystemAdminEmail = ReadingLists::SystemAdminEmail();
        my $olddsn = ReadingLists::dsn();
        $olddsn =~ /database=(.+);user=(.+);password=(.+);mysql/;
        $DBname = $1;
        $DBuser = $2;
        $DBpassword = $3;
	$got_pm=1;
    }
}
if (!$got_pm) { print "\nUsing default factory settings.\n" };

# Intro blurb
print STDOUT <<"EndOfIntro";
                  Loughborough University Reading List System

                          I N S T A L L A T I O N
                         -------------------------

This is the installation system for the reading list system developed
at Loughborough University Library.  This program will examine your
machine and ask you questions to work out where it should place the
various components of the reading lists system on your computer.  If
you get stuck on a question, don't worry - you can run this program
again as many times as you like.  Also many of the questions will have
default answers in square brackets; typing carriage return will give
you the default.

EndOfIntro

#
# Ask user if they'd like help
#
my $setup_wiz = 
  ask("Would like you like to be offered tips on answering the questions?",
        "y", "q");

print "\n";

#
# Check if we have the required Perl modules to hand.
#
check_required_mods();

#
# Ask loads of questions about how the user wants the system to be setup
#
if($setup_wiz) {
    print STDOUT <<"EndOfTip";
The core code for the reading list system is in a couple of Perl modules.
These can be placed into any directory in the filesystem.  Please
supply a directory where you would like to place these.  If the directory
does not exist, it will be created (along with any required parent 
directories).

EndOfTip
}
$ReadingListsLibraryDir = ask("Where would you like the Perl modules to live?",
			      $ReadingListsLibraryDir, "d");
if(! -d $ReadingListsLibraryDir) {
    system("mkdir","-p",$ReadingListsLibraryDir);
    if(! -d $ReadingListsLibraryDir) {
	die("Can't create $ReadingListsLibraryDir - bailing out!\n");
    }
}
print "\n";

if($setup_wiz) {
    print STDOUT <<"EndOfTip";
There a number of "backend" scripts that can be used by the reading
list system administrator/cron.  These can live in any directory in
the filesystem.  Please supply a directory where you would like to
place these.  If the directory does not exist, it will be created
(along with any required parent directories).

EndOfTip
}
$BackendBinDir = ask("Where would you like the backend scripts to live?",
    $BackendBinDir, "d");
if(! -d $BackendBinDir) {
    system("mkdir","-p",$BackendBinDir);
    if(! -d $BackendBinDir) {
	die("Can't create $BackendBinDir - bailing out!\n");
    }
}
print "\n";

if($setup_wiz) {
    print STDOUT <<"EndOfTip";
The format of module codes varies from institution to institution.  This
reading list system allows you to specify what format you wish to use by
supplying a specification string.  Module codes are typically made up from
year components, departmental codes, stage of course codes and opaque 
identifiers.  In the format string we represent these using:

  Y = year digit
  D = department code letter
  S = stage code letter
  X = opaque letter

For example Loughborough's format string is "YYDDSXXX" so we can have module
codes like "01LBA123", "02COC321", etc.  The module code format is used
in the system to provide basic validation of module codes entered by users
and also to allow users to "skip" certain bits of data entry about the module
if it can be deduced from the module code (eg Loughborough's user don't get
asked about years, departments or course stages as these can be worked out
from the module code.  If your module code is just a meaningless identifier
just use the appropriate number of X's in this string.

EndOfTip
}
$ModuleCodeFormat = ask("What module code format do you need?",
			$ModuleCodeFormat, "s");
print "\n";

if($setup_wiz) {
    print STDOUT <<"EndOfTip";
The reading list system can be intergrated with existing Library OPACs.  One
option is a link to the main WebOPAC entry page.  For this you need to 
supply your WebOPAC URL.

EndOfTip
}
$OpacURL = ask("What is the URL of your WebOPAC?",
	       $OpacURL, "s");
print "\n";

if($setup_wiz) {
    print STDOUT <<"EndOfTip";
Many WebOPACs allow you to "deep link" into them using an ISBN.  This
option asks you for a URL for your WebOPAC that allows the system to
do that.  It is used to link directly into the WebOPAC from reading
lists, so that end users can check on availability of interesting
works, reserve them, etc.  In the URL, please put "%s" where you would
normally put the ISBN.

EndOfTip
}
$OpacISBNURL = ask("What is the URL of your WebOPAC ISBN display?",
	       $OpacISBNURL, "s");
print "\n";

if($setup_wiz) {
    print STDOUT <<"EndOfTip";

The main interface for this reading list system, for both end users,
staff adding information and librarians, is provided using CGI web
forms.  Please enter the base URI for this host.  The default should
be right for this host most of the time but you may wish to have an
alias for the machine (a so called DNS "CNAME") so that you can move
it to another host in the future and/or share the host with other
services.

EndOfTip
}
$HostURI = ask("What is the base URI for your reading list web server?",
	       $HostURI, "s");
print "\n";

if($setup_wiz) {
    print STDOUT <<"EndOfTip";

The reading list system interfaces with the catalogue of your OPAC
using Z39.50.  Please enter the hostname of the machine providing
your OPAC's Z39.50 services.

EndOfTip
}
$Z3950Hostname = ask("What is the hostname for your Z39.50 server?",
		     $Z3950Hostname, "s");
print "\n";

if($setup_wiz) {
    print STDOUT <<"EndOfTip";

The Z39.50 server can be listening on any port number.  Many servers
listen on port 210 but Aleph uses 9909 for some reason.  Check with
your OPAC system documentation/vendor for details of the port your
OPAC is listening on.  Then enter the Z39.50 server's port number here.

EndOfTip
}
$Z3950Port = ask("What is the port of your Z39.50 server?",
		 $Z3950Port, "n");
print "\n";

if($setup_wiz) {
    print STDOUT <<"EndOfTip";

The Z39.50 might well be serving out multiple databases.  We need to
know which database to use to lookup catalogue records.  Please enter
the OPAC database name here.

EndOfTip
}
$Z3950DBName = ask("What is the Z39.50 database name?",
		   $Z3950DBName, "s");
print "\n";

if($setup_wiz) {
    print STDOUT <<"EndOfTip";

Some Z39.50 server may require a username in order to connect to them.
If your Z39.50 server is one of these, please enter the username here.

EndOfTip
}
$Z3950Username = ask("What is the username for the Z39.50 connection?",
		     $Z3950Username, "s");
print "\n";

if($setup_wiz) {
    print STDOUT <<"EndOfTip";

Some Z39.50 server may require a password in order to connect to them.
If your Z39.50 server is one of these, please enter the password here.

EndOfTip
}
$Z3950Password = ask("What is the password for the Z39.50 connection?",
		     $Z3950Password, "s");
print "\n";

if($setup_wiz) {
    print STDOUT <<"EndOfTip";

There are three different sets of CGI scripts in this reading list system;
user CGI scripts (used by end users, eg students), admin CGI scripts (used
by people creating and maintaining reading lists, eg academic staff,
administrative staff, librarians, etc) and librarian CGI scripts (used
by library staff).  Each of these sets of CGI scripts can live in a different
directory in the UNIX filesystem and can thus be protected differently
in your web server.  For example you might allow anyone on the Internet
to use the user scripts, only staff who can authenticate against a campus
staff mail server to use the admin scripts and only a special group of
library staff to use the librarian scripts.  Creation of groups
and access rights is something you will need to configure in your web server
before running the reading list system - this installation script does
NOT set that up for you (it depends on the web server you use and where
you will be getting authentication information from - see your web server
documentation for details).

First of all we need to know where in the UNIX filesystem you would like
the user CGI scripts to be placed (full path to the directory please!).

EndOfTip
}
$UserCGIBinDir = ask("Where to place the user CGI scripts in the filesystem?",
	       $UserCGIBinDir, "d");
if(! -d $UserCGIBinDir) {
    system("mkdir","-p",$UserCGIBinDir);
    if(! -d $UserCGIBinDir) {
	die("Can't create $UserCGIBinDir - bailing out!\n");
    }
}
print "\n";

if($setup_wiz) {
    print STDOUT <<"EndOfTip";

Now we need to know what the path in the URLs for the user scripts should
be.  This depends on what path name you have configured your web server 
with in order to reference the $UserCGIBinDir UNIX directory.

EndOfTip
}
$UserCGI = ask("What is the URI path to the end user CGI scripts?",
	       $UserCGI, "s");
print "\n";

if($setup_wiz) {
    print STDOUT <<"EndOfTip";

Next we need the full UNIX path to the directory that will hold the admin
CGI scripts (used by staff creating and editing reading lists).

EndOfTip
}
$AdminCGIBinDir = ask("Where to place the admin CGI scripts in the filesystem?",
		      $AdminCGIBinDir, "d");
if(! -d $AdminCGIBinDir) {
    system("mkdir","-p",$AdminCGIBinDir);
    if(! -d $AdminCGIBinDir) {
	die("Can't create $AdminCGIBinDir - bailing out!\n");
    }
}
print "\n";

if($setup_wiz) {
    print STDOUT <<"EndOfTip";

Now we need to know what the path in the URLs for the admin scripts should
be.  This depends on what path name you have configured your web server 
with in order to reference the $AdminCGIBinDir UNIX directory.

EndOfTip
}
$AdminCGI = ask("What is the URI path to the admin CGI scripts?",
		$AdminCGI, "s");
print "\n";

if($setup_wiz) {
    print STDOUT <<"EndOfTip";

Next we need the full UNIX path to the directory that will hold the librarian
CGI scripts (used by library staff to manage the reading list system).

EndOfTip
}
$LibrarianCGIBinDir = ask("Where to place the librarian CGI scripts in the filesystem?",
		      $LibrarianCGIBinDir, "d");
if(! -d $LibrarianCGIBinDir) {
    system("mkdir","-p",$LibrarianCGIBinDir);
    if(! -d $LibrarianCGIBinDir) {
	die("Can't create $LibrarianCGIBinDir - bailing out!\n");
    }
}
print "\n";

if($setup_wiz) {
    print STDOUT <<"EndOfTip";

Now we need to know what the path in the URLs for the librarian
scripts should be.  This depends on what path name you have configured
your web server with in order to reference the $LibrarianCGIBinDir UNIX
directory.

EndOfTip
}
$LibrarianCGI = ask("What is the URI path to the library staff CGI scripts?",
		    $LibrarianCGI, "s");
print "\n";

if($setup_wiz) {
    print STDOUT <<"EndOfTip";

The CGI scripts make use of Cascading Style Sheets to help with the layout
of the resulting HTML pages.  A sample CSS stylesheet file is included with 
this distribution (called readinglists.css).  We need to know which directory
to put this file in (that is a directory that your web server can see and 
serve files out from) and also what path to access it via on the web.

EndOfTip
}
$CSSDir = ask("Where to place the CSS stylesheet file in the filesystem?",
		    $CSSDir, "s");
print "\n";

if($setup_wiz) {
    print STDOUT <<"EndOfTip";

Next we need the path that the to the CSS file via the web server.

EndOfTip
}
$CSSPath = ask("What is the URI path to the CSS stylesheet file?",
		    $CSSPath, "s");
print "\n";

if($setup_wiz) {
    print STDOUT <<"EndOfTip";

Some of the scripts in this system can email librarians with details of
reading list changes, problems, etc.  In case a librarian registers
with no email address, please enter a default library email address here.
This may also be used for some end user queries.

EndOfTip
}
$DefaultLibraryEmail = ask("Email address for general library queries?",
    $DefaultLibraryEmail, "m");
$DefaultLibraryEmail =~ s/\@/\\\@/;
print "\n";

if($setup_wiz) {
    print STDOUT <<"EndOfTip";

Sometimes users may need to email the system administrator of this reading
list system.  Please enter that person/group's email address here.

EndOfTip
}
$SystemAdminEmail = ask("Email address of the reading list system administrator?",
    $SystemAdminEmail, "m");
$SystemAdminEmail =~ s/\@/\\\@/;
print "\n";

if($setup_wiz) {
    print STDOUT <<"EndOfTip";

The MySQL database underlying this reading list can be called anything
you like.  Please enter the name you wish to use here.  The database
structure and some initial data will be created for you as part of this
installation process if it does not already exist.

EndOfTip
}
$DBname = ask("What is the name of the reading list database in MySQL?",
	      $DBname, "s");
print "\n";

if($setup_wiz) {
    print STDOUT <<"EndOfTip";
The MySQL database $DBname must be owned by and accessible to a particular
database user.  This does NOT have to correspond to a UNIX login id.  You'll
need to create this user (this script doesn't do it for you) using some
SQL such as:

    grant CREATE, SELECT, INSERT, UPDATE, DELETE on * to rl\@localhost 
      identified by 'oghma';

where 'rl' is the username and 'oghma' is the password, whilst logged
in as a database user with "GRANT" rights, such as the 'root' database
user, using an SQL command interface such as:

    mysql -u root -pyourrootpassword

You now have to tell this script what the username you used was.

EndOfTip
} 

$DBuser = ask("What MySQL user should connect to the $DBname database?",
	      $DBuser, "s"); print "\n";

if($setup_wiz) {
    print STDOUT <<"EndOfTip";

Now we need to know the password for the database user ("oghma" in the 
example in the last step).

EndOfTip
}
$DBpassword = ask("What password should $DBuser use to connect to the $DBname database?\n",
		  $DBpassword, "s");
print "\n";

my $go_for_it =
  ask("Are you sure that you wish to proceed with the installation?",
        "y", "q");
if($go_for_it ne "y") {
    exit;
}


# Install the library modules
replacevars("lib", $ReadingListsLibraryDir);

# Install the backend scripts
replacevars("bin", $BackendBinDir);

# Install the user CGI scripts
replacevars("cgi-bin/readinglists",
	    $UserCGIBinDir);

# Install the admin CGI scripts
replacevars("cgi-bin/readinglistsadmin",
	    $AdminCGIBinDir);

# Install the librarian CGI scripts
replacevars("cgi-bin/readinglistsadmin/librarians",
	    $LibrarianCGIBinDir);

# Install the CSS file
if(-f "$CSSDir/readinglists.css") {
    print <<"EndOfTip";
There already appears to be a file called $CSSDir/readinglists.css on the
machine.  Leaving it where it is - this will be used by the CSS styling
on the HTML generated by the various CGI scripts.

EndOfTip
} else {
    open(SOURCE,"readinglists.css") ||
	 die "Can't open 'readinglists.css' for reading - bailing out!\n";
    open(TARGET,">$CSSDir/readinglists.css") ||
         die "Can't open '$CSSDir/readinglists.css' for writing - bailing out!\n";
    while(my $line=<SOURCE>) {
        print TARGET $line;
    }
    close TARGET;
    close SOURCE;
}

# Create the reading list database schema/initial data for the database we've
# been told to use.
open(OLDSCHEMA,"schema.sql") ||
    die "Can't read schema.sql: $!\n";
open(NEWSCHEMA,">localschema.sql") || 
    die "Can't write to localschema.sql: $!\n";
while (my $line = <OLDSCHEMA>) {
    $line =~ s/create database readinglist/create database $DBname/;
    $line =~ s/use readinglist/use $DBname/;
    print NEWSCHEMA $line;
}
close(NEWSCHEMA);
close(OLDSCHEMA);
open(OLDSCHEMA,"initial.sql") ||
    die "Can't read initial.sql: $!\n";
open(NEWSCHEMA,">localinitial.sql") || 
    die "Can't write to localinitial.sql: $!\n";
while (my $line = <OLDSCHEMA>) {
    $line =~ s/use readinglist/use $DBname/;
    print NEWSCHEMA $line;
}
close(NEWSCHEMA);
close(OLDSCHEMA);
my @dbs = DBI->data_sources("mysql");
my $dbhere = 0;
foreach my $thisdb (@dbs) {
    if($thisdb eq "DBI:mysql:$DBname") {
	$dbhere = 1;
	last;
    }
}

if(!$dbhere) {
    print STDOUT <<"EndOfTip";

Creating reading list database called $DBname.  If you see any errors between
here and the "DB creation scripts finished" line then the database will
probably not have been created properly.  You will need to check that the
database server is running, you have created a database user with the
right name and password and that you can successfully log in as that user
manually.  You can then other rerun this install script with the right
information or else "source" the localschema.sql and localinitial.sql
files in the MySQL command interpreter.

EndOfTip
    system("mysql","-u",$DBuser,"-p$DBpassword","-e",
	   "source localschema.sql");
    system("mysql","-u",$DBuser,"-p$DBpassword","-e",
	   "source localinitial.sql");
    print STDOUT "\n\nDB creation scripts finished\n\n";
    print STDOUT <<"EndOfTip";

The initial database has a single department called "Library" with a
department code of "LB".  There are two initial users created; libby is
a library and aker is an academic.  Both of them are owners of an initial
demo reading list with the module code "01LBA001" (which is assuming that
you are using Loughborough style module codes in the "YYDDSXXX" format;
if not you will have to pop into the MySQL database and tweak the module_code
field of the readinglist table appropriately).  If you create libby and
aker users in your webserver (eg using htpasswd with Apache) you should then
be able to access the editing tools and play with the demo readinglists.

Enjoy LORLS - we hope you find it useful.  Please report bugs and feature
suggestions to G.P.Brewerton\@lboro.ac.uk and J.P.Knight\@lboro.ac.uk.


EndOfTip
} else {
    print STDOUT "\n\nDatabase $DBname already exists - not touching!\n\n";
}

exit;

#
# Subroutine to ask a question to a user
#
sub ask {
    my($prompt, $default, $type) = @_;

    my $satisfied = "no";
    my $answer = "";
    while ($satisfied eq "no") {
	print "$prompt [$default] ";
	$answer = <STDIN>;
	$answer =~ s/\s+$//;;

        # use default value if they just press return
	$answer = $default if $answer =~ /^(\s+|)$/;

        # simple checks
	$satisfied = "yes", last if ($type eq "d" || $type eq "s");
	$satisfied = "yes", last if ($type eq "D" && -d "$answer");
	$satisfied = "yes", last if ($type eq "m" && $answer =~ /\@/);
	$satisfied = "yes", last if ($type eq "n" && $answer =~ /\d+/);
	$satisfied = "yes", last if ($type eq "q" && $answer =~ /^[yn]/i && 
                                     ($answer = lc($answer)));
	$satisfied = "yes", last if ($type eq "S" && length($answer));
	$satisfied = "yes", last if ($type eq "X" && -x "$answer");

        # flag errors
	print "*** The directory \"$answer\" doesn't exist! ***\n"
	    if ($type eq "D");
	print "*** Email addresses must contain an \@ sign! ***\n"
	    if ($type eq "m");
	print "*** The answer must be a number! ***\n"
	    if ($type eq "n");
	print "*** You must answer yes or no! ***\n"
	    if ($type eq "q");
	print "*** You must answer this question! ***\n"
	    if ($type eq "S");
	print "*** The program must exist and be runnable! ***\n"
	    if ($type eq "X");
    } 
    return ($answer ? $answer : $default);
}

#
# Subroutine to do the install copying and variable replacement in scripts
#
sub replacevars {
    my($distdir,$targetdir) = @_;
    print STDOUT "Installing files from $distdir\n" .
                 "                   to $targetdir...\n\n";
    opendir(DISTDIR,$distdir) || die "Can't open $distdir in distribution!\n";
    while(my $thisfile = readdir(DISTDIR)) {
        next if(-d "$distdir/$thisfile");
        next if($thisfile =~ /~$/);
	print STDOUT "Processing $thisfile ";
        open(SOURCE,"$distdir/$thisfile") ||
	    die "Can't open '$distdir/$thisfile' for reading - bailing out!\n";
        open(TARGET,">$targetdir/$thisfile") ||
	    die "Can't open '$targetdir/$thisfile' for writing - bailing out!\n";
        while(my $line = <SOURCE>) {
            $line =~ s/use lib \("\/usr\/local\/ReadingLists"\);/use lib (\"$ReadingListsLibraryDir\");/  && print "1";
	    $line =~ s/return\("YYDDSXXX"\);/return(\"$ModuleCodeFormat\");/  && print "2";
	    $line =~ s/return\("http:\/\/158.125.80.5\/ALEPH"\);/return(\"$OpacURL\");/ && print "3";
	    $line =~ s/return\(sprintf\("http:\/\/jackanory.lut.ac.uk\/ALEPH\/-\/ext-find\?base=lbo01&find=WIS=%s",\$isbn\)\);/return(sprintf(\"$OpacISBNURL\",\$isbn));/ && print "4";
	    $line =~ s/return\("https:\/\/bookworm.lboro.ac.uk\/"\);/return(\"$HostURI\");/ && print "5";
	    $line =~ s/return\("opac.lboro.ac.uk"\);/return(\"$Z3950Hostname\");/ && print "6";
	    $line =~ s/return\("9929"\);/return(\"$Z3950Port\");/ && print "7";
	    $line =~ s/return\("LBO01"\);/return(\"$Z3950DBName\");/ && print "8";
	    $line =~ s/return\(""\); \# Z3950Username/return(\"$Z3950Username\"); \# Z3950Username/ && print "9";
	    $line =~ s/return\(""\); \# Z3950Password/return(\"$Z3950Password\"); \# Z3950Password/ && print "A";
	    $line =~ s/return\("\/rl"\);/return(\"$UserCGI\");/ && print "B";
	    $line =~ s/return\("\/rl\/admin"\);/return(\"$AdminCGI\");/ && print "C";
	    $line =~ s/return\("\/rl\/admin\/librarians"\);/return(\"$LibrarianCGI\");/ && print "D";
	    $line =~ s/return\("library\\\@lboro.ac.uk"\);/return(\"$DefaultLibraryEmail\");/ && print "E";
	    $line =~ s/return\("g.p.brewerton\\\@lboro.ac.uk"\);/return(\"$SystemAdminEmail\");/ && print "F";
	    $line =~ s/return "DBI:mysql:database=readinglist;user=root;password=mysqlpass;"/return \"DBI:mysql:database=$DBname;user=$DBuser;password=$DBpassword;\"/ && print "G";
	    $line =~ s/href='\/readinglists.css'/href='$CSSPath\/readinglists.css'/ && print "H";
	    if($line =~ /sub ReadingListsLibraryDir \{/) {
		print TARGET $line;
		$line = <SOURCE>;
		$line = "  return(\"$ReadingListsLibraryDir\");";
	    }
	    if($line =~ /sub BackendBinDir \{/) {
		print TARGET $line;
		$line = <SOURCE>;
		$line = "  return(\"$BackendBinDir\");";
	    }
	    if($line =~ /sub UserCGIBinDir \{/) {
		print TARGET $line;
		$line = <SOURCE>;
		$line = "  return(\"$UserCGIBinDir\");";
	    }
	    if($line =~ /sub AdminCGIBinDir \{/) {
		print TARGET $line;
		$line = <SOURCE>;
		$line = "  return(\"$AdminCGIBinDir\");";
	    }
	    if($line =~ /sub LibrarianCGIBinDir \{/) {
		print TARGET $line;
		$line = <SOURCE>;
		$line = "  return(\"$LibrarianCGIBinDir\");";
	    }
	    print TARGET $line;
	}
	close(TARGET);
	close(SOURCE);
        system("chmod","a+rx","$targetdir/$thisfile");
	print " - done\n";
    }
    closedir(DISTDIR);
    print "\nDone\n\n";
}

sub check_required_mods {
    my $requiredmods;
    $requiredmods = { 
	'MARC' => 'http://bookworm.lboro.ac.uk/dist/MARC-1.15.tar.gz',
	'Net::Z3950' => 'http://bookworm.lboro.ac.uk/dist/Net-Z3950-0.44.tar.gz',
    };

    print "Checking for required Perl modules...\n";
    my $notfound = 0;
    foreach my $thismod (keys %{$requiredmods}) {
	print "Checking for $thismod... ";
        if(eval"require $thismod;") {
	    print "found OK\n";
	} else {
	    print "Missing\n";
	    $notfound = 1;
	    my $url = $requiredmods->{$thismod};
	    print << "EndOfMsg";

You will need to install the $thismod Perl module before you can install
this system.  You can get a copy from CPAN or alternatively from
<URL:$url>.

EndOfMsg
        }
    }
    print "\n";
    if($notfound) {
	print <<"EndOfMsg";
Some required Perl modules were missing; aborting this installation run.

Please install the modules listed above and rerun this script.

EndOfMsg
        exit;
    }
}
