#
# Perl object representing an item in a reading list
#
# Authors: J.P.Knight@lboro.ac.uk and G.P.Brewerton@lboro.ac.uk
#
# $Id: ReadingListsItem.pm,v 1.49 2005/10/03 10:47:13 jon Exp jon $
#
#   Copyright (C) 2001-2002 Loughborough University Library
#   Authors: Jon Knight and Gary Brewerton
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

package ReadingListsItem;
use lib ("/usr/local/ReadingLists");
use strict;
use Carp;
use LWP;
use MARC::Record;
use ReadingLists;

my %item = (
    material_id     => undef,
    modulecode     => undef,
    authors        => undef,
    title          => undef,
    control_no     => undef,
    edition        => undef,
    publisher      => undef,
    pubdate        => undef,
    partauthor     => undef,
    parttitle      => undef,
    volume         => undef,
    issue          => undef,
    pages          => undef,
    indicative     => 0,
    keytext        => 0,
    purchase       => 0,
    held           => 0,
    rec_long_loan  => 0,
    rec_week_loan  => 0,
    rec_short_loan => 0,
    rec_referemce  => 0,
    url            => undef,
    notes          => undef,
    openurl        => undef,
    private_note   => undef,
	    );

sub new {
    my $that = shift;
    my $class = ref($that) || $that;
    my %params = @_;
    my $self = { 
      %item,
      };
    bless $self, $class;

    $self->{db} = DBI->connect(ReadingLists::dsn) || return($self);

    # If an item number is specified try to get the data from the database
    $self->{material_id} = $params{material_id} || undef;
    $self->{modulecode} = $params{modulecode} || undef;
#    if(defined($self->{material_id}) && 
#       defined($self->{modulecode})) {
    if(defined($self->{material_id}) ) {
      my $material_id = $self->{material_id};
      my $sql = "select * from material where material_id = $material_id";
      my $sh = $self->{db}->prepare($sql);
      $sh->execute;
      if($sh) {
	  my $r = $sh->fetchrow_hashref;
          my %row1 = %{$r};
          $self->{authors} = $row1{"authors"};
	  $self->{title} = $row1{"title"};
	  $self->{control_no} = $row1{"control_no"};
          $self->{edition} = $row1{"edition"};
	  $self->{publisher} = $row1{"publisher"};
	  $self->{pubdate} = $row1{"pubdate"};
	  $self->{url} = $row1{"url"};
	  $self->{openurl} = $row1{"openurl"};
	  $self->{pages} = $row1{"pages"};
	  $self->{partauthor} = $row1{"partauthor"};
	  $self->{parttitle} = $row1{"parttitle"};
	  $self->{volume} = $row1{"volume"};
	  $self->{issue} = $row1{"issue"};
          $self->{held} = $row1{"held"};
	  $self->{keytext} = $row1{"keytext"};
	  $self->{purchase} = $row1{"purchase"};
          $self->{indicative} = $row1{"indicative"};
	  $self->{rec_long_loan} = $row1{"rec_long_loan"};
	  $self->{rec_week_loan} = $row1{"rec_week_loan"};
	  $self->{rec_short_loan} = $row1{"rec_short_loan"};
          $self->{rec_reference} = $row1{"rec_reference"};
          $self->{private_note} = $row1{"private_note"};
          $self->{created_time} = $row1{"created_time"};
          $self->{created_by} = $row1{"created_by"};
          $self->{modified_time} = $row1{"modified_time"};
          $self->{modified_by} = $row1{"modified_by"};
      }
      $sh->finish;
      # Now get the note data (if any).
      $self->{notes} = $self->getnote;
    }

    $self->{authors} = $params{authors} unless(!defined($params{authors}));
    $self->{authors} =~ s/\s*$// if(defined($self->{authors}));
    $self->{title} = $params{title} unless(!defined($params{title}));
    $self->{title} =~ s/\s*$// if(defined($self->{title}));
    $self->{control_no} = $params{control_no}
      unless(!defined($params{control_no}));
    $self->{edition} = $params{edition} unless(!defined($params{edition}));
    $self->{publisher} = $params{publisher} 
      unless(!defined($params{publisher}));
    $self->{publisher} =~ s/\s*$// if(defined($self->{publisher}));
    $self->{pubdate} = $params{pubdate} unless(!defined($params{pubdate}));
    $self->{pubdate} =~ s/\s*$// if(defined($self->{pubdate}));
    $self->{partauthor} = $params{partauthor}
      unless(!defined($params{partauthor}));
    $self->{partauthor} =~ s/\s*$// if(defined($self->{partauthor}));
    $self->{parttitle} = $params{parttitle}
      unless(!defined($params{parttitle}));
    $self->{parttitle} =~ s/\s*$// if(defined($self->{parttitle}));
    $self->{volume} = $params{volume} unless(!defined($params{volume}));
    $self->{volume} =~ s/\s*$// if(defined($self->{volume}));
    $self->{issue} = $params{issue} unless(!defined($params{issue}));
    $self->{issue} =~ s/\s*$// if(defined($self->{issue}));
    $self->{pages} = $params{pages} unless(!defined($params{pages}));
    $self->{pages} =~ s/\s*$// if(defined($self->{pages}));
    $self->{keytext} = $params{keytext} unless(!defined($params{keytext}));
    $self->{purchase} = $params{purchase} unless(!defined($params{purchase}));
    $self->{rec_long_loan} = $params{rec_long_loan} 
      unless(!defined($params{rec_long_loan}));
    $self->{rec_week_loan} = $params{rec_week_loan} 
      unless(!defined($params{rec_week_loan}));
    $self->{rec_short_loan} = $params{rec_short_loan} 
      unless(!defined($params{rec_short_loan}));
    $self->{url} = $params{url} unless(!defined($params{url}));
    $self->{url} =~ s/\s*$// if(defined($self->{url}));
    $self->{notes} = $params{notes} unless(!defined($params{notes}));
    $self->{notes} =~ s/\s*$// if(defined($self->{notes}));
    $self->{private_note} = $params{private_note}
      unless(!defined($params{private_note}));
    $self->{private_note} =~ s/\s*$// if(defined($self->{private_note}));
   
    return $self;
}

#
# Subroutine to be called as an object destructed
#
sub DESTROY {
    my $self = shift;
    $self->{db}->disconnect;
}


sub RetrieveDataFromISBN {
    use Net::Z3950;
    my $self = shift;
    my $class = ref($self) || croak "$self is not an object";
    my %params = @_;

    my $Found = 0;
    return($Found) if(!defined($self->{control_no}) 
		      || $self->{control_no} eq "");
    my $isbn = $self->{control_no};

    my $zhost = ReadingLists::Z3950Hostname;
    $zhost = $params{'zhost'} if(defined($params{'zhost'}));
    my $zport = ReadingLists::Z3950Port;
    $zport = $params{'zport'} if(defined($params{'zport'}));
    my $zdbname = ReadingLists::Z3950DBName; 
    $zdbname = $params{'zdbname'} if(defined($params{'zdbname'}));
    my $zuser = ReadingLists::Z3950Username;
    $zuser = $params{'zuser'} if(defined($params{'zuser'}));
    my $zpassword = ReadingLists::Z3950Password;
    $zpassword = $params{'zpassword'} if(defined($params{'zpassword'}));
    my $zconnlife = ReadingLists::Z3950ConnectionLifespan;
    $zconnlife = $params{'zconnlife'} if(defined($params{'zconnlife'}));

    # Check to see if we have an issn
    if($isbn =~ /^(\d\d\d\d)(\d\d\d[\d|x])$/i) {
	$isbn = "$1-$2";
    }
    # Get rid of a leading "ISBN" or "ISSN" string
    $isbn =~ s/^IS[SB]N//i;

    if (($isbn =~ /^\d\d\d\d-\d\d\d[\d|x]$/i)
        ||($isbn =~/^\d\d\d\d\d\d$/)
        ||($isbn =~/^s\d\d\d\d\d\d\d..$/)) {
        #ISSN
    } else {
        #ISBN
#        $isbn =~ s/\-^$//g;
        $isbn =~ s/ //g;
        $isbn =~ s/-//g;
    }
    my $redo = 1;
    if(!defined($self->{z3950conncount})) {
	$self->{z3950conncount} = -1;
    }
    while($redo) {
	$redo = 0;
	if(!defined($self->{z3950conn}) || 
		    $self->{z3950conncount} == -1 ||
		    $self->{z3950conncount} > $zconnlife) {
	    $self->{z3950conn} = 
		new Net::Z3950::Connection($zhost,$zport,
					   databaseName => "$zdbname",
					   user => "$zuser",
					   password => "$zpassword"
					   )
		or die "can't connect: $!";
	    $self->{z3950conncount} = 0;
	}

	my $rs = $self->{z3950conn}->search(-prefix => $isbn);
	$rs->option(elementSetName => "") unless(!defined($rs));

	if((defined($rs)) && $rs && (my $n = $rs->size() > 0)) {
	    $rs->option(preferredRecordSyntax => 
                        Net::Z3950::RecordSyntax::USMARC);
	    for (my $i = 0; $i < $n; $i++) {
		my $rec = $rs->record($i+1);
		if (!defined $rec) {
                    warn "record", $i+1, ": error #", $rs->errcode(),
		      " (", $rs->errmsg(), "): ", $rs->addinfo(), "\n";
		    next;
		}
		my $mrec = MARC::Record->new_from_usmarc($rec->rawdata());

#		warn "MARC record contains: " . $mrec->as_formatted() . "\n";

                # Hack to deal with knackered ISBNs at Loughborough
		my $tmpisbn = $isbn;
		$tmpisbn =~ s/--$/-/;

                # Some records have more than one 022 field so we need to
                # check all of them.
                my @fields = ();
		if(defined($mrec->fields('020'))) {
		    push @fields, $mrec->field('020');
		}
		if(defined($mrec->fields('022'))) {
		    push @fields, $mrec->field('022');
		}
		if(defined($mrec->fields('500'))) {
		    push @fields, $mrec->field('500');
		}
		my $matched = 0;
		foreach my $field (@fields) {
		    if(defined($field->subfield('a')) && 
		       $field->subfield('a') =~ /\s*$isbn/i) {
			$matched = 1;
			last;
		    }
		    if(defined($field->subfield('b')) && 
		       $field->subfield('b') =~ /\s*$isbn/i) {
			$matched = 1;
			last;
		    }
		}

		if((defined($mrec->field('001')->data()) && (
                    ($mrec->field('001')->data() =~ /\s*$isbn/i) ||
		    ($mrec->field('001')->data() =~ /\s*-$isbn/i) ||
		    ($mrec->field('001')->data() =~ /\s*$tmpisbn/i) ||
		    ($mrec->field('001')->data() =~ /\s*-$isbn-/i))) ||
 		   $matched) {
		    $Found = 1;
		    $self->{held} = 1;
		    my $author =  "";
		    my $title = "";
		    my $publisher = "";
		    my $volume = "";
		    my $part = "";
		    my $edition = "";
		    my $partauthor = "";
		    my $pubdate = "";
		    my $authorjoin = "";

		    if($mrec->author()) {
			my $tmpauthor = $mrec->author();
			$tmpauthor =~ s/[^a-zA-Z0-9\)]$//;
			$author .= $authorjoin.$tmpauthor;
			$authorjoin= ", ";
		    }
		    if($mrec->title_proper()) {
			$title = $mrec->title_proper();
			$title =~ s/\/\s*$//;
			if($mrec->field('245')->subfield('b')) {
			    $title .= " ".$mrec->field('245')->subfield('b');
			    $title =~ s/[^a-zA-Z0-9\)]$//;
			}
			$title =~ s/:(\w)/: $1/g;
		    }
		    if($mrec->edition()) {
			$edition = $mrec->edition();
			$edition =~ s/[^a-zA-Z0-9\)]$//;
		    }
		    if($mrec->publication_date()) {
			$pubdate = $mrec->publication_date();
			$pubdate =~ s/[^a-zA-Z0-9\)]$//;
			$pubdate = "" if($pubdate eq "NULL");
			if($mrec->field('260')->subfield('b')) {
			    $publisher = $mrec->field('260')->subfield('b');
			    $publisher =~ s/[^a-zA-Z0-9\)]$//;
			}
		    }
		    if($mrec->field('700') &&
		       defined($mrec->field('700')->subfield('a'))) {
			my $tmpauthor = $mrec->field('700')->subfield('a');
			$tmpauthor =~ s/[^a-zA-Z0-9\)]$//;
			$author .= $authorjoin.$tmpauthor;
			$authorjoin= ", ";
		    }
		    $self->{authors} = $author if($author);
		    $self->{partauthor} = $partauthor if($partauthor);
		    $self->{volume} = $volume if($volume);
		    $self->{title} = $title if($title);
		    $self->{title} =~ s/\s*$//;
		    $self->{edition} = $edition if($edition);
		    $self->{publisher} = $publisher if($publisher);
		    $self->{pubdate} = $pubdate if($pubdate);
		}
		last if($Found);
	    }
	} else {
	    if(defined($self->{z3950conn}->errcode()) &&
               $self->{z3950conn}->errcode() eq "100") {
		$redo = 1;
	    }
	}
	$self->{z3950conncount}++;
	if($self->{z3950conncount} > $zconnlife) {
	    $self->{z3950conn}->close();
	    $self->{z3950conn}->DESTROY;
	}
    }
    return($Found);
}

#
# Update the record for this material item in the readinglist database
#
sub Update {
    my $self = shift;
    my $class = ref($self) || croak "$self is not an object";
    my $reterr = "";

    if(!defined($self->{modulecode})) {
      return("No module code was provided");
    }
    my $modulecode = $self->{db}->quote($self->{modulecode});

    # Get the list_id of this reading list.  We need this later to update
    # sequence information.  If we can't find a list_id something has gone
    # badly wrong so don't try to update or create anything.
    my $sql = "select list_id from readinglist ".
      "where module_code = $modulecode";
    my $sh = $self->{db}->prepare($sql);
    $sh->execute;
    if($sh->rows < 1){
      $sh->finish;
      return("Attempted to update a reading list item for which no list_id " .
	     "was found - aborting update!\n");
    }
    my ($listid) = $sh->fetch->[0];
    $sh->finish;

    my $now;
    chop($now = `date +'%Y-%m-%d %H:%M:%S'`);
    $now = $self->{db}->quote($now);
    my $user = $ENV{REMOTE_USER};
    unless (defined($user)) { chop($user = `whoami`); }
    $user = $self->{db}->quote($user);

    # To stop clueless people from over writing an existing entry with
    # a completely new one and thus having incorrect holding information,
    # we'll do a sneaky ISBN lookup.
    if(defined($self->{control_no}) && $self->{control_no} ne "") {
        my $tmprl = new ReadingListsItem;
        $tmprl->{control_no} = $self->{control_no};
        if($tmprl->RetrieveDataFromISBN) {
	    $self->{held} = 1;
	} else {
	    $self->{held} = 0;
	}
    } else {
        $self->{held} = 0;
    }

    if(defined($self->{material_id})) {
      # We're updating an existing item's record in the database.  The first
      # thing to do is to make a copy of the existing record entry in the
      # archive_material table so that librarians can get a historical view
      # of what changes have been made to an item.
      $sql = "insert into archive_material " .
        "(material_id, title, authors, edition, publisher, pubdate, " .
        "control_no, parttitle, partauthor, volume, issue, pages, " .
        "indicative, keytext, purchase, held, rec_long_loan, " .
        "rec_week_loan, rec_short_loan, rec_reference, url, created_time, " .
        "created_by, modified_time, modified_by, openurl, private_note) " .
        "select material_id, title, authors, edition, publisher, pubdate, " .
        "control_no, parttitle, partauthor, volume, issue, pages, " .
        "indicative, keytext, purchase, held, rec_long_loan, " .
        "rec_week_loan, rec_short_loan, rec_reference, url, created_time, " .
        "created_by, modified_time, modified_by, openurl, private_note " .
        " from material where " .
        "(material_id = " . $self->{material_id} . ")";
      $sh = $self->{db}->prepare($sql);
      $sh->execute;
      $sh->finish;

      # Now we can update the real material table with the new details
      my $material_id = $self->{db}->quote($self->{material_id});
      my $title = $self->{db}->quote($self->{title});
      my $authors = $self->{db}->quote($self->{authors});
      my $edition = $self->{db}->quote($self->{edition});
      my $publisher = $self->{db}->quote($self->{publisher});
      my $pubdate = $self->{db}->quote($self->{pubdate});
      $pubdate = "NULL" if(defined($self->{pubdate}) && $self->{pubdate} eq "NULL");
      my $control_no = $self->{db}->quote($self->{control_no});
      my $parttitle = $self->{db}->quote($self->{parttitle});
      my $partauthor = $self->{db}->quote($self->{partauthor});
      my $volume = $self->{db}->quote($self->{volume});
      my $issue = $self->{db}->quote($self->{issue});
      my $pages = $self->{db}->quote($self->{pages});
      my $indicative = $self->{db}->quote($self->{indicative});
      my $keytext = $self->{db}->quote($self->{keytext});
      my $purchase = $self->{db}->quote($self->{purchase});
      my $held = $self->{db}->quote($self->{held});
      my $rec_long_loan = $self->{db}->quote($self->{rec_long_loan});
      my $rec_week_loan = $self->{db}->quote($self->{rec_week_loan});
      my $rec_short_loan = $self->{db}->quote($self->{rec_short_loan});
      my $rec_reference = $self->{db}->quote($self->{rec_reference});
      my $url = $self->{db}->quote($self->{url});
      my $private_note = $self->{db}->quote($self->{private_note});
      $sql = "update material set " . 
        "title=$title, authors=$authors, edition=$edition, ".
        "publisher=$publisher, pubdate=$pubdate, control_no=$control_no, " .
        "parttitle=$parttitle, partauthor=$partauthor, volume=$volume, " .
        "issue=$issue, pages=$pages, indicative=$indicative, " .
        "keytext=$keytext, purchase=$purchase, held=$held, " .
        "rec_long_loan=$rec_long_loan, rec_week_loan=$rec_week_loan, " .
        "rec_short_loan=$rec_short_loan, rec_reference=$rec_reference, " .
        "url=$url, private_note=$private_note, " .
        "modified_time=$now, modified_by=$user " .
	"where material_id=$material_id";
      $sh = $self->{db}->prepare($sql);
      $sh->execute;
      $sh->finish;
      my $note_id = updatenotes($self);
    } else {
      # This is a new item to add to this reading list.
      my $material_id = $self->{db}->quote($self->{material_id});
      my $title = $self->{db}->quote($self->{title});
      my $authors = $self->{db}->quote($self->{authors});
      my $edition = $self->{db}->quote($self->{edition});
      my $publisher = $self->{db}->quote($self->{publisher});
      my $pubdate = $self->{db}->quote($self->{pubdate});
      my $control_no = $self->{db}->quote($self->{control_no});
      my $parttitle = $self->{db}->quote($self->{parttitle});
      my $partauthor = $self->{db}->quote($self->{partauthor});
      my $volume = $self->{db}->quote($self->{volume});
      my $issue = $self->{db}->quote($self->{issue});
      my $pages = $self->{db}->quote($self->{pages});
      my $indicative = $self->{db}->quote($self->{indicative});
      my $keytext = $self->{db}->quote($self->{keytext});
      my $purchase = $self->{db}->quote($self->{purchase});
      my $held = $self->{db}->quote($self->{held});
      my $rec_long_loan = $self->{db}->quote($self->{rec_long_loan});
      my $rec_week_loan = $self->{db}->quote($self->{rec_week_loan});
      my $rec_short_loan = $self->{db}->quote($self->{rec_short_loan});
      my $rec_reference = $self->{db}->quote($self->{rec_reference});
      my $url = $self->{db}->quote($self->{url});
      my $private_note = $self->{db}->quote($self->{private_note});
      $sql = "insert into material " .
        "(material_id, title, authors, edition, publisher, pubdate, " .
        "control_no, parttitle, partauthor, volume, issue, pages, " .
        "indicative, keytext, purchase, held, rec_long_loan, " .
        "rec_week_loan, rec_short_loan, rec_reference, url, private_note, ".
        "created_time, created_by, modified_time, modified_by) " .
	"values (0, $title, $authors, $edition, $publisher, " .
        "$pubdate, $control_no, $parttitle, $partauthor, $volume, $issue, " .
        "$pages, $indicative, $keytext, $purchase, $held, $rec_long_loan, ". 
        "$rec_week_loan, $rec_short_loan, $rec_reference, $url, " .
        "$private_note, $now, $user, $now, $user)";
      $sh = $self->{db}->prepare($sql);
      $sh->execute;
      $sh->finish;

      # Have to now get the new material_id number for this item
      # This makes use of the MySQL auto_increment ID.  Need to be changed
      # if another database is used instead as it is non-standard.  Note that
      # this might need to have locking turn on so that we make sure that we
      # get the material ID from the item we just entered if multiple items
      # are being entered at once.
      $sql = "select LAST_INSERT_ID() from material";
      $sh = $self->{db}->prepare($sql);
      $sh->execute;
      $self->{material_id} = $sh->fetch->[0];
      $sh->finish;

      # Insert the notes into the annotations table and get the resulting
      # note_id
      my $note_id = insertnotes($self);

    }

    # Update the last_item_update field in the reading list table.  This is
    # used by some of our code to quickly scan for readinglists that have
    # been altered after a given datetime
    $sql = "update readinglist set last_item_update=$now, ".
        "last_item_update_by=$user ".
	"where module_code = $modulecode";

    $sh = $self->{db}->prepare($sql);
    $sh->execute;

    # OK we should have updated the database now.  As a check we can
    # pull the data back out of the database.
    $sql = "select * from material where material_id = " . 
	$self->{material_id};
    $sh = $self->{db}->prepare($sql);
    $sh->execute;
    my $r = $sh->fetchrow_hashref;
    my %r = %{$r};
    $sh->finish;
    $self->{title} = $r{title};
    $self->{authors} =$r{authors};
    $self->{edition} = $r{edition};
    $self->{publisher} =$r{publisher};
    $self->{pubdate} =$r{pubdate};
    $self->{control_no} = $r{control_no};
    $self->{parttitle} = $r{parttitle};
    $self->{partauthor} =$r{partauthor};
    $self->{volume} = $r{volume};
    $self->{issue} = $r{issue};
    $self->{pages} = $r{pages};
    $self->{indicative} = $r{indicative};
    $self->{keytext} = $r{keytext};
    $self->{purchase} = $r{purchase};
    $self->{held} = $r{held};
    $self->{rec_long_loan} = $r{rec_long_loan};
    $self->{rec_week_loan} = $r{rec_week_loan};
    $self->{rec_short_loan} = $r{rec_short_loan};
    $self->{rec_reference} = $r{rec_reference};
    $self->{url} = $r{url};
    $self->{private_note} = $r{private_note};
    $self->{notes} = $self->getnote;

    return($reterr);
}

#
# Subroutine to get changes for an item
#
sub changes {
  my $self = shift;
  my ($material_id, $startdate) = @_;

  my $db = $self->{db};
  return() unless(defined($db)); 

  # Get the current date/time
  my $now;
  chop($now = `date +'%Y-%m-%d %H:%M:%S'`);
  $now = $db->quote($now);
  $startdate = "2000-01-01 00:00:00" if($startdate eq "");

  # Get the current material details
  my @result = ();
  my $sql = "select * from material where " .
      "material_id = $material_id";
  my $sh = $db->prepare($sql);
  my %lastitem;
  $sh->execute;
  if($sh->rows < 1) {
    return(());
  } else {
    my $r = $sh->fetchrow_hashref;
    my %row = %{$r};
    foreach my $thiskey (keys %row) {
      $lastitem{$thiskey} = $row{$thiskey};
    }
  }
  $lastitem{edit_number} = 0;

  # Get the changes
  $sql = "select * from archive_material where " .
    "material_id = $material_id and modified_time > '$startdate' " .
	"order by modified_time desc";
  $sh = $db->prepare($sql);
  $sh->execute;
  if($sh->rows < 1) {
    @result = ();
  } else {
    my $count = 0;
    my $rowcount = 0;
    while(my $r = $sh->fetchrow_hashref) {
      my %row = %{$r};
      foreach my $thiskey (keys %row) {
        if($row{$thiskey} ne $lastitem{$thiskey} && 
          ($thiskey ne "modified_by" && $thiskey ne "modified_time" &&
           $thiskey ne "created_by" && $thiskey ne "created_time" &&
           $thiskey ne "edit_number")) {
	  $result[$count]->{date} = $lastitem{modified_time};
	  $result[$count]->{attribute} = $thiskey;
	  $result[$count]->{newvalue} = $lastitem{$thiskey};
          $result[$count]->{oldvalue} = $row{$thiskey};
          $result[$count]->{editor} = $lastitem{modified_by};
          $count++;  
          $lastitem{$thiskey} = $row{$thiskey};
        }
      }
      $rowcount++;
    }
  }
  return(@result);
}

# Subroutine to get note for an item.  The notes are held separately 
# from the rest of the item as they may be standalone.  We need to
# look in the sequence table for this reading list to get a pointer
# to them.
sub getnote {
  my $self = shift;
  my $notetext = undef;

#  warn "Entering getnote()...\n";

  # Get the id of this reading list
  my $modulecode = $self->{db}->quote($self->{modulecode});
  my $sql = "select list_id from readinglist ".
     "where module_code = $modulecode";
#  print "<em>sql = $sql</em>\n";
  my $sh = $self->{db}->prepare($sql);
  $sh->execute;
  if($sh->rows < 1){
    $sh->finish;
    return(undef) ;
  }
  my ($listid) = $sh->fetch->[0];
  $sh->finish;

  # Get the ID of a note (if any) related to the item in the sequence table
  my $material_id = $self->{material_id};
  $sql = "select note_id from sequence where list_id = $listid and " .
    "material_id = $material_id";
  $sh = $self->{db}->prepare($sql);
  $sh->execute;
  my $noteid = undef;
  if($sh->rows > 0) {
    $noteid = $sh->fetch->[0];
  }
  $sh->finish;

  # If there was such an ID, retrieve the contents from the annotiation table
  if(defined($noteid)) {
    $sql = "select content from annotation where note_id = $noteid";
    $sh = $self->{db}->prepare($sql);
    $sh->execute;
    $notetext = $sh->fetch->[0];
  }

  $sh->finish;
#  warn "Returning from getnote() with '$notetext'\n";
  return($notetext);
}

# Subroutine to insert note for an item.  The notes are held separately 
# from the rest of the item as they may be standalone.  We need to
# look in the sequence table for this reading list to get a pointer
# to them.
sub insertnotes {
  my $self = shift;

#  warn "Entering insertnote()...\n";

  # Get the id of this reading list
  my $material_id = $self->{material_id};
  my $modulecode = $self->{db}->quote($self->{modulecode});
  my $sql = "select list_id from readinglist ".
     "where module_code = $modulecode";
  my $sh = $self->{db}->prepare($sql);
  $sh->execute;
  if($sh->rows < 1){
    $sh->finish;
    return(undef) ;
  }
  my ($listid) = $sh->fetch->[0];
  $sh->finish;

  # Get the user doing this insert and the current time
  my $now;
  chop($now = `date +'%Y-%m-%d %H:%M:%S'`);
  $now = $self->{db}->quote($now);
  my $user = $ENV{REMOTE_USER};
  unless (defined($user)) { chop($user = `whoami`); }
  $user = $self->{db}->quote($user);

  # Insert the new note into the note table.
  my $notetext = $self->{db}->quote($self->{notes});
  $sql = "insert into annotation (note_id, content, created_time, " .
    "created_by, modified_time, modified_by) values (0, $notetext, " .
    "$now, $user, $now, $user)";
  $sh = $self->{db}->prepare($sql);
  $sh->execute;

  # Get the note_id of the newly created annotation
  $sql = "select LAST_INSERT_ID() from annotation";
  $sh = $self->{db}->prepare($sql);
  $sh->execute;
  my $note_id = $sh->fetch->[0];
  $sh->finish;

  # Work out what sequence ID relates to this reading list and the maximum
  # rank in the table.  We need the latter because we're going to stick 
  # the new note at the end of the sequence.
  $sql = "select max(sequence_id) from sequence where " .
    "list_id = $listid";
  $sh = $self->{db}->prepare($sql);
  $sh->execute;
  my $seqid = undef;
  my $rank = undef;
  if($sh) {
      my @row1 = $sh->fetchrow_array;
      $seqid = $row1[0];
      $seqid++;
  }
  $sh->finish;
  $sql = "select max(rank) from sequence where " .
    "list_id = $listid";
  $sh = $self->{db}->prepare($sql);
  $sh->execute;
  if($sh) {
      my @row1 = $sh->fetchrow_array;
      $rank = $row1[0];
      $rank++;
  }
  $sh->finish;

  # What the f*ck?  If there's no sequence id available we need to get the
  # maximum sequence id and create a new sequence with an id one bigger than
  # that.  This can only happen if someone creates a reading list and starts
  # by creating just an annotation, rather than adding a full item.
  if(!defined($seqid)) {
    $sql = "select max(sequence_id) from sequence";
    $sh = $self->{db}->prepare($sql);
    $sh->execute;
    if($sh) {
      my @row1 = $sh->fetchrow_array;
      $seqid = $row1[0];
      $seqid++;
    } else {
      $seqid = 0;
    }
    $rank = 0; 
  }

  # Insert the information about this item in the sequence table.  We
  # need to get a ranking for this item - it'll be one higher than the
  # previous highest value to ensure that it is at the end of the list
  $sql = "insert into sequence (list_id, material_id, " .
    "note_id, rank, created_time, created_by, modified_time, " .
    "modified_by, number_of_edits) values ($listid, " .
    "$material_id, $note_id, $rank, $now, $user, $now, $user, 0)";
  $sh = $self->{db}->prepare($sql);
  $sh->execute;

  $sh->finish;
#  warn "Returning from insertnotes()\n";
  return($note_id);
}

# Subroutine to update note for an item.  The notes are held separately 
# from the rest of the item as they may be standalone.  We need to
# look in the sequence table for this reading list to get a pointer
# to them.
sub updatenotes {
  my $self = shift;

#  warn "Entering updatenotes()...\n";

  # Get the user doing this insert and the current time
  my $now;
  chop($now = `date +'%Y-%m-%d %H:%M:%S'`);
  $now = $self->{db}->quote($now);
  my $user = $ENV{REMOTE_USER};
  unless (defined($user)) { chop($user = `whoami`); }
  $user = $self->{db}->quote($user);

  # Get the id of this reading list
  my $modulecode = $self->{db}->quote($self->{modulecode});
  my $notetext = $self->{db}->quote($self->{notes});
  my $sql = "select list_id from readinglist ".
     "where module_code = $modulecode";
  my $sh = $self->{db}->prepare($sql);
  $sh->execute;
  if($sh->rows < 1){
    $sh->finish;
#    warn "Couldn't find the module's readinglist!\n";
    return(undef) ;
  }
  my ($listid) = $sh->fetch->[0];
  $sh->finish;

  # Get the ID of a note (if any) related to the item in the sequence table
  my $material_id = $self->{material_id};
  $sql = "select note_id from sequence where list_id = $listid and " .
    "material_id = $material_id";
  $sh = $self->{db}->prepare($sql);
  $sh->execute;
  my $noteid = undef;
  if($sh->rows > 0) {
    $noteid = $sh->fetch->[0];
  }
  $sh->finish;

  # If there was such an ID, update the contents, modification info and
  # number of edits
  if(defined($noteid)) {
    $sql = "select number_of_edits from sequence where note_id = $noteid";
    $sh = $self->{db}->prepare($sql);
    $sh->execute;
    my $edits = 0;
    if($sh->rows > 0) {
      $edits = $sh->fetch->[0];
    }
    $edits++;
    $sh->finish;
    $sql = "update annotation set content=$notetext, modified_time=$now, " .
      "modified_by=$user" .
      "where note_id=$noteid";
    $sh = $self->{db}->prepare($sql);
    $sh->execute;
    $sh->finish;
    $sql = "update sequence set number_of_edits=$edits ".
      "where note_id = $noteid";
    $sh = $self->{db}->prepare($sql);
    $sh->execute;
    $sh->finish;
  } else {
    # Hey, we can't update a note that isn't already there!  So insert it
    # as a new note instead.  Everything is probably ballsed up if we get
    # to this point
    $noteid = insertnotes($self);
  }

#  warn "Returning from updatenotes()\n";
  return($noteid);
}

#
# Cheeky autoloader that means that the data structure can be altered
# via method calls.
#
sub AUTOLOAD {
    my $self = shift;
    my $class = ref($self) || croak "$self is not an object";
    no strict; # PITA - $AUTOLOAD is a package global that strict doesn't
               # know about as it is created on the fly.
    my $name = $AUTOLOAD;
    use strict;
    $name =~ s/.*://; # strip fully=qualified portion
    unless (exists $self->{$name}) {
	croak "Can't access '$name' field in object of class $class";
    }
    if(@_) {
	return $self->{$name} = shift;
    } else {
	return $self->{$name};
    }
}

1;

__END__

=head1 NAME

ReadingListsItem - Module implementing a reading list item object and methods

=head1 REVISION

$Revision: 1.49 $

=head1 SYNOPSIS

use ReadingListsItem;

$item = new ReadingListsItem(I<params>);

$found = $item->RetrieveDataFromISBN;

$result = $item->Update;

$notetext = $item->getnote;

$item->insertnotes;

$item->updatenotes;

=head1 DESCRIPTION

This module implements a reading list item object and provides methods for
creating and manipuulating these objects.  The reading list item data
itself is currently stored in a relational database (specifically a MySQL
database) but the object methodology employed by this module will allow
the actual representation of this data to be changed in the future without
affecting scripts that make use of it.

As well as the methods detailed below, the perl module has an AUTOLOADer that
allows access to the individual fields in the reading list item.  These are
"methods" are called:

=over 4

=item material_id - an ID for this material ID

=item modulecode - the code for the module whose reading list

=item authors - Authors of the main work

=item title - Title of the main work

=item control_no - Control number (ISBN or ISSN) of the main work

=item edition - Edition of the main work

=item publisher - Publisher of the main work

=item pubdate - Publication date of the main work

=item partauthor - Authors of part of a work

=item parttitle - Title of part of a work

=item volume - Volume that a part work is in

=item issue - Issue that a part work is in

=item pages - Pages that a part work is in

=item indicative - Flag whether the item is indicative or not

=item keytext - Flag whether the item is a key text or not

=item purchase - Flag whether the item is recommended for purchase

=item held - Flag whether the resource is held in the OPAC or not

=item rec_long_loan - Number of long loan copies of the item that the reading
list owner has recommended be made available

=item rec_week_loan - Number of week loan copies of the item that the reading
list owner has recommended be made available

=item rec_short_loan - Number of short loan copies of the item that the 
reading list owner has recommended be made available

=item rec_referemce - Number of reference copies of the item that the
reading list owner has recommended be made available

=item url - URL of an electronic resource related to this item

=item notes - Text of an attached annotation

=back

=head1 METHODS

=head2 new

This method creates a new material item object.  It creates the Perl
data structure for the item data and opens a connection to the reading
list database (using ReadingLists::dsn).  The method can also be
supplied with a hash of parameters.  The keys of this hash can be any
of the AUTOLOADed method names listed above.  If both a material_id
and a modulecode are supplied, the object retrieves the data from an
existing material item from the database.  It then overwrites any
attributes of that material item that have been specified in the
parameter hash (eg author, title, etc).  Note that this overwriting
only effects the Perl ReadingListsItem object, not the data in the
backend database.  If you want to update that, you need to then call
the Update method detailed below.

=head2 RetrieveDataFromISBN

This method attempts to retrieve some of the data for a material item
by looking its control number (ISBN or ISSN) up in a remote database.
At the moment only the Talis OPAC database at Loughborough is
supported.  Any data retrieved overwrites existing data of the same
type in the Perl object (ie authors, titles, etc) whilst leaving other
data untouched (modulecode, material_id, rec_long_loan, etc).

=head2 Update

This method takes a username and a modulecode and updates a material
item in the backend database.  If the item does not already exist, it
is created and inserted into the end of the sequence for the reading
list (based on the modulecode that is in this object - if there is no
modulecode specified this method does not update the database and
returns an error string), otherwise it is updated.  If the operation
is successful this method returns an empty string, otherwise it
returns a string detailing the error experienced.  If it is successful it
also rereads the data from the backend database and reloads this into the
Perl object.  This make sure that what you subsequently retrieve from
the Perl object is what is actually stored in the database.

=head2 getnote

Returns the text of a note attached to the material item.  This is similar
to the getnote method in the ReadingLists object except that it does not
require a note ID to be specified and can only get the text of a note
that is part of a material item, not a standalone annotation.

=head2 insertnotes

This method inserts the note text held in the Perl item object into the
database.  Its is used by the Update method above and should not really 
need to be used by applications.

=head2 updatenotes

This method updates the note text in the database from the note text
held in the Perl object.  Its is used by the Update method above and
should not really need to be used by applications.

=head1 BUGS

Probably loads.  The RetrieveDataFromISBN is currently rather tied to
Talis and should probably be made more general purpose (use a separate
module per data source perhaps?).  There are also some methods in the
ReadingLists module that might want to be in this module instead
(reorderitem and deleteitem for example).

=head1 COPYRIGHT

This code is free software and is distributed under the Gnu General
Public License (GPL).  A copy of the GPL should have been supplied along
with this software.  If you don't have a copy you can get one from
<URL:http://www.gnu.org>.

This code developed by the Systems Team in the Pilkington Library at
Loughborough University, England.  The code as delivered "as is"; no
warrenty is made for its fitness of purpose or against claims of loss
or damage resulting from its use.

=head1 AUTHORS

Jon Knight <J.P.Knight@lboro.ac.uk> and 
Gary Brewerton <G.P.Brewerton@lboro.ac.uk>
